@@ -1,9 +1,10 @@
+/**
+ * @license
+ * Copyright 2013 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
+
 /*
- * Copyright 2013 The Emscripten Authors.  All rights reserved.
- * Emscripten is available under two separate licenses, the MIT license and the
- * University of Illinois/NCSA Open Source License.  Both these licenses can be
- * found in the LICENSE file.
- *
  * EMSCRIPTEN GLFW 2.x-3.x emulation.
  * It tries to emulate the behavior described in
  * http://www.glfw.org/docs/latest/
@@ -35,10 +36,8 @@
  */
 
 var LibraryGLFW = {
-  $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser'],
-  $GLFW: {
-
-    Window: function(id, width, height, title, monitor, share) {
+  $GLFW_Window__docs: '/** @constructor */',
+  $GLFW_Window: function(id, width, height, title, monitor, share) {
       this.id = id;
       this.x = 0;
       this.y = 0;
@@ -80,6 +79,12 @@ var LibraryGLFW = {
       this.userptr = null;
     },
 
+  $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser', '$GLFW_Window',
+#if FILESYSTEM
+    , '$FS'
+#endif
+  ],
+  $GLFW: {
     WindowFromId: function(id) {
       if (id <= 0 || !GLFW.windows) return null;
       return GLFW.windows[id - 1];
@@ -187,7 +192,7 @@ var LibraryGLFW = {
         case 0xDC:return 92; // DOM_VK_BACKSLASH -> GLFW_KEY_BACKSLASH
         case 0xDD:return 93; // DOM_VK_CLOSE_BRACKET -> GLFW_KEY_RIGHT_BRACKET
         case 0xC0:return 94; // DOM_VK_BACK_QUOTE -> GLFW_KEY_GRAVE_ACCENT
-        
+
 #if USE_GLFW == 2
         //#define GLFW_KEY_SPECIAL      256
         case 0x1B:return (256+1); // DOM_VK_ESCAPE -> GLFW_KEY_ESC
@@ -356,11 +361,11 @@ var LibraryGLFW = {
       if (charCode == 0 || (charCode >= 0x00 && charCode <= 0x1F)) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii') }}}(GLFW.active.charFunc, charCode, 1);
+      {{{ makeDynCall('vii', 'GLFW.active.charFunc') }}}(charCode, 1);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('vii') }}}(GLFW.active.charFunc, GLFW.active.id, charCode);
+      {{{ makeDynCall('vii', 'GLFW.active.charFunc') }}}(GLFW.active.id, charCode);
 #endif
     },
 
@@ -378,12 +383,12 @@ var LibraryGLFW = {
       if (!GLFW.active.keyFunc) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii') }}}(GLFW.active.keyFunc, key, status);
+      {{{ makeDynCall('vii', 'GLFW.active.keyFunc') }}}(key, status);
 #endif
 
 #if USE_GLFW == 3
       if (repeat) status = 2; // GLFW_REPEAT
-      {{{ makeDynCall('viiiii') }}}(GLFW.active.keyFunc, GLFW.active.id, key, keyCode, status, GLFW.getModBits(GLFW.active));
+      {{{ makeDynCall('viiiii', 'GLFW.active.keyFunc') }}}(GLFW.active.id, key, keyCode, status, GLFW.getModBits(GLFW.active));
 #endif
     },
 
@@ -428,11 +433,11 @@ var LibraryGLFW = {
       if (event.target != Module["canvas"] || !GLFW.active.cursorPosFunc) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii') }}}(GLFW.active.cursorPosFunc, Browser.mouseX, Browser.mouseY);
+      {{{ makeDynCall('vii', 'GLFW.active.cursorPosFunc') }}}(Browser.mouseX, Browser.mouseY);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('vidd') }}}(GLFW.active.cursorPosFunc, GLFW.active.id, Browser.mouseX, Browser.mouseY);
+      {{{ makeDynCall('vidd', 'GLFW.active.cursorPosFunc') }}}(GLFW.active.id, Browser.mouseX, Browser.mouseY);
 #endif
     },
 
@@ -456,7 +461,7 @@ var LibraryGLFW = {
       if (event.target != Module["canvas"] || !GLFW.active.cursorEnterFunc) return;
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('vii') }}}(GLFW.active.cursorEnterFunc, GLFW.active.id, 1);
+      {{{ makeDynCall('vii', 'GLFW.active.cursorEnterFunc') }}}(GLFW.active.id, 1);
 #endif
     },
 
@@ -466,7 +471,7 @@ var LibraryGLFW = {
       if (event.target != Module["canvas"] || !GLFW.active.cursorEnterFunc) return;
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('vii') }}}(GLFW.active.cursorEnterFunc, GLFW.active.id, 0);
+      {{{ makeDynCall('vii', 'GLFW.active.cursorEnterFunc') }}}(GLFW.active.id, 0);
 #endif
     },
 
@@ -491,11 +496,11 @@ var LibraryGLFW = {
       if (!GLFW.active.mouseButtonFunc) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii') }}}(GLFW.active.mouseButtonFunc, eventButton, status);
+      {{{ makeDynCall('vii', 'GLFW.active.mouseButtonFunc') }}}(eventButton, status);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('viiii') }}}(GLFW.active.mouseButtonFunc, GLFW.active.id, eventButton, status, GLFW.getModBits(GLFW.active));
+      {{{ makeDynCall('viiii', 'GLFW.active.mouseButtonFunc') }}}(GLFW.active.id, eventButton, status, GLFW.getModBits(GLFW.active));
 #endif
     },
 
@@ -518,7 +523,7 @@ var LibraryGLFW = {
       if (!GLFW.active || !GLFW.active.scrollFunc || event.target != Module['canvas']) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vi') }}}(GLFW.active.scrollFunc, GLFW.wheelPos);
+      {{{ makeDynCall('vi', 'GLFW.active.scrollFunc') }}}(GLFW.wheelPos);
 #endif
 
 #if USE_GLFW == 3
@@ -532,7 +537,7 @@ var LibraryGLFW = {
         sy = event.deltaY;
       }
 
-      {{{ makeDynCall('vidd') }}}(GLFW.active.scrollFunc, GLFW.active.id, sx, sy);
+      {{{ makeDynCall('vidd', 'GLFW.active.scrollFunc') }}}(GLFW.active.id, sx, sy);
 #endif
 
       event.preventDefault();
@@ -587,11 +592,11 @@ var LibraryGLFW = {
       if (!GLFW.active.windowSizeFunc) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii') }}}(GLFW.active.windowSizeFunc, GLFW.active.width, GLFW.active.height);
+      {{{ makeDynCall('vii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.width, GLFW.active.height);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii') }}}(GLFW.active.windowSizeFunc, GLFW.active.id, GLFW.active.width, GLFW.active.height);
+      {{{ makeDynCall('viii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
     },
 
@@ -601,38 +606,10 @@ var LibraryGLFW = {
       if (!GLFW.active.framebufferSizeFunc) return;
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii') }}}(GLFW.active.framebufferSizeFunc, GLFW.active.id, GLFW.active.width, GLFW.active.height);
+      {{{ makeDynCall('viii', 'GLFW.active.framebufferSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
     },
 
-    requestFullscreen: function() {
-      var RFS = Module["canvas"]['requestFullscreen'] ||
-                Module["canvas"]['mozRequestFullScreen'] ||
-                Module["canvas"]['webkitRequestFullScreen'] ||
-                (function() {});
-      RFS.apply(Module["canvas"], []);
-    },
-
-    requestFullScreen: function() {
-      err('GLFW.requestFullScreen() is deprecated. Please call GLFW.requestFullscreen instead.');
-      GLFW.requestFullScreen = function() {
-        return GLFW.requestFullscreen();
-      }
-      return GLFW.requestFullscreen();
-    },
-
-    exitFullscreen: function() {
-      Browser.exitFullscreen();
-    },
-
-    cancelFullScreen: function() {
-      err('GLFW.cancelFullScreen() is deprecated. Please call GLFW.exitFullscreen instead.');
-      GLFW.cancelFullScreen = function() {
-        return GLFW.exitFullscreen();
-      }
-      return GLFW.exitFullscreen();
-    },
-
     getTime: function() {
       return _emscripten_get_now() / 1000;
     },
@@ -679,7 +656,7 @@ var LibraryGLFW = {
               };
 
               if (GLFW.joystickFunc) {
-                {{{ makeDynCall('vii') }}}(GLFW.joystickFunc, joy, 0x00040001); // GLFW_CONNECTED
+                {{{ makeDynCall('vii', 'GLFW.joystickFunc') }}}(joy, 0x00040001); // GLFW_CONNECTED
               }
             }
 
@@ -697,7 +674,7 @@ var LibraryGLFW = {
               console.log('glfw joystick disconnected',joy);
 
               if (GLFW.joystickFunc) {
-                {{{ makeDynCall('vii') }}}(GLFW.joystickFunc, joy, 0x00040002); // GLFW_DISCONNECTED
+                {{{ makeDynCall('vii', 'GLFW.joystickFunc') }}}(joy, 0x00040002); // GLFW_DISCONNECTED
               }
 
               _free(GLFW.joys[joy].id);
@@ -765,7 +742,7 @@ var LibraryGLFW = {
 
       event.preventDefault();
 
-#if '$FS' in addedLibraryItems
+#if FILESYSTEM
       var filenames = allocate(new Array(event.dataTransfer.files.length*4), 'i8*', ALLOC_NORMAL);
       var filenamesArray = [];
       var count = event.dataTransfer.files.length;
@@ -788,7 +765,7 @@ var LibraryGLFW = {
           var data = e.target.result;
           FS.writeFile(path, new Uint8Array(data));
           if (++written === count) {
-            {{{ makeDynCall('viii') }}}(GLFW.active.dropFunc, GLFW.active.id, count, filenames);
+            {{{ makeDynCall('viii', 'GLFW.active.dropFunc') }}}(GLFW.active.id, count, filenames);
 
             for (var i = 0; i < filenamesArray.length; ++i) {
               _free(filenamesArray[i]);
@@ -806,8 +783,7 @@ var LibraryGLFW = {
       for (var i = 0; i < count; ++i) {
         save(event.dataTransfer.files[i]);
       }
-
-#endif // '$FS' in addedLibraryItems
+#endif // FILESYSTEM
 
       return false;
     },
@@ -824,14 +800,14 @@ var LibraryGLFW = {
       if (!win) return null;
       var prevcbfun = win.windowSizeFunc;
       win.windowSizeFunc = cbfun;
-     
+
 #if USE_GLFW == 2
       // As documented in GLFW2 API (http://www.glfw.org/GLFWReference27.pdf#page=22), when size
       // callback function is set, it will be called with the current window size before this
       // function returns.
       // GLFW3 on the over hand doesn't have this behavior (https://github.com/glfw/glfw/issues/62).
       if (!win.windowSizeFunc) return null;
-      {{{ makeDynCall('vii') }}}(win.windowSizeFunc, win.width, win.height);
+      {{{ makeDynCall('vii', 'win.windowSizeFunc') }}}(win.width, win.height);
 #endif
 
       return prevcbfun;
@@ -940,8 +916,13 @@ var LibraryGLFW = {
         wy = win.y;
       }
 
-      setValue(x, wx, 'i32');
-      setValue(y, wy, 'i32');
+      if (x) {
+        setValue(x, wx, 'i32');
+      }
+      
+      if (y) {
+        setValue(y, wy, 'i32');
+      }
     },
 
     setWindowPos: function(winid, x, y) {
@@ -961,8 +942,13 @@ var LibraryGLFW = {
         wh = win.height;
       }
 
-      setValue(width, ww, 'i32');
-      setValue(height, wh, 'i32');
+      if (width) {
+        setValue(width, ww, 'i32');
+      }
+      
+      if (height) {
+        setValue(height, wh, 'i32');
+      }
     },
 
     setWindowSize: function(winid, width, height) {
@@ -971,9 +957,9 @@ var LibraryGLFW = {
 
       if (GLFW.active.id == win.id) {
         if (width == screen.width && height == screen.height) {
-          GLFW.requestFullscreen();
+          Browser.requestFullscreen();
         } else {
-          GLFW.exitFullscreen();
+          Browser.exitFullscreen();
           Browser.setCanvasSize(width, height);
           win.width = width;
           win.height = height;
@@ -983,17 +969,19 @@ var LibraryGLFW = {
       if (!win.windowSizeFunc) return;
 
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii') }}}(win.windowSizeFunc, width, height);
+      {{{ makeDynCall('vii', 'win.windowSizeFunc') }}}(width, height);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii') }}}(win.windowSizeFunc, win.id, width, height);
+      {{{ makeDynCall('viii', 'win.windowSizeFunc') }}}(win.id, width, height);
 #endif
     },
 
     createWindow: function(width, height, title, monitor, share) {
       var i, id;
-      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] !== null; i++);
+      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] !== null; i++) {
+        // no-op
+      }
       if (i > 0) throw "glfwCreateWindow only supports one window at time currently";
 
       // id for window
@@ -1003,19 +991,21 @@ var LibraryGLFW = {
       if (width <= 0 || height <= 0) return 0;
 
       if (monitor) {
-        GLFW.requestFullscreen();
+        Browser.requestFullscreen();
       } else {
         Browser.setCanvasSize(width, height);
       }
 
       // Create context when there are no existing alive windows
-      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] == null; i++);
+      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] == null; i++) {
+        // no-op
+      }
       if (i == GLFW.windows.length) {
         var contextAttributes = {
           antialias: (GLFW.hints[0x0002100D] > 1), // GLFW_SAMPLES
           depth: (GLFW.hints[0x00021005] > 0),     // GLFW_DEPTH_BITS
           stencil: (GLFW.hints[0x00021006] > 0),   // GLFW_STENCIL_BITS
-          alpha: (GLFW.hints[0x00021004] > 0)      // GLFW_ALPHA_BITS 
+          alpha: (GLFW.hints[0x00021004] > 0)      // GLFW_ALPHA_BITS
         }
 #if OFFSCREEN_FRAMEBUFFER
         // TODO: Make GLFW explicitly aware of whether it is being proxied or not, and set these to true only when proxying is being performed.
@@ -1028,7 +1018,7 @@ var LibraryGLFW = {
       if (!Module.ctx) return 0;
 
       // Get non alive id
-      var win = new GLFW.Window(id, width, height, title, monitor, share);
+      var win = new GLFW_Window(id, width, height, title, monitor, share);
 
       // Set window to array
       if (id - 1 == GLFW.windows.length) {
@@ -1047,7 +1037,7 @@ var LibraryGLFW = {
 
 #if USE_GLFW == 3
       if (win.windowCloseFunc)
-        {{{ makeDynCall('vi') }}}(win.windowCloseFunc, win.id);
+        {{{ makeDynCall('vi', 'win.windowCloseFunc') }}}(win.id);
 #endif
 
       GLFW.windows[win.id - 1] = null;
@@ -1119,6 +1109,10 @@ var LibraryGLFW = {
     window.addEventListener("keypress", GLFW.onKeyPress, true);
     window.addEventListener("keyup", GLFW.onKeyup, true);
     window.addEventListener("blur", GLFW.onBlur, true);
+    Module["canvas"].addEventListener("touchmove", GLFW.onMousemove, true);
+    Module["canvas"].addEventListener("touchstart", GLFW.onMouseButtonDown, true);
+    Module["canvas"].addEventListener("touchcancel", GLFW.onMouseButtonUp, true);
+    Module["canvas"].addEventListener("touchend", GLFW.onMouseButtonUp, true);
     Module["canvas"].addEventListener("mousemove", GLFW.onMousemove, true);
     Module["canvas"].addEventListener("mousedown", GLFW.onMouseButtonDown, true);
     Module["canvas"].addEventListener("mouseup", GLFW.onMouseButtonUp, true);
@@ -1142,6 +1136,10 @@ var LibraryGLFW = {
     window.removeEventListener("keypress", GLFW.onKeyPress, true);
     window.removeEventListener("keyup", GLFW.onKeyup, true);
     window.removeEventListener("blur", GLFW.onBlur, true);
+    Module["canvas"].removeEventListener("touchmove", GLFW.onMousemove, true);
+    Module["canvas"].removeEventListener("touchstart", GLFW.onMouseButtonDown, true);
+    Module["canvas"].removeEventListener("touchcancel", GLFW.onMouseButtonUp, true);
+    Module["canvas"].removeEventListener("touchend", GLFW.onMouseButtonUp, true);
     Module["canvas"].removeEventListener("mousemove", GLFW.onMousemove, true);
     Module["canvas"].removeEventListener("mousedown", GLFW.onMouseButtonDown, true);
     Module["canvas"].removeEventListener("mouseup", GLFW.onMouseButtonUp, true);
@@ -1343,17 +1341,26 @@ var LibraryGLFW = {
       ww = win.width;
       wh = win.height;
     }
-
-    setValue(width, ww, 'i32');
-    setValue(height, wh, 'i32');
+    
+    if (width) {
+      setValue(width, ww, 'i32');
+    }
+    
+    if (height) {
+      setValue(height, wh, 'i32');
+    }
   },
 
   glfwIconifyWindow: function(winid) {
-    GLFW.iconifyWindow(winid);
+#if ASSERTIONS
+    warnOnce('glfwIconifyWindow is not implemented');
+#endif
   },
 
   glfwRestoreWindow: function(winid) {
-    GLFW.restoreWindow(winid);
+#if ASSERTIONS
+    warnOnce('glfwRestoreWindow is not implemented');
+#endif
   },
 
   glfwShowWindow: function(winid) {},
@@ -1642,11 +1649,15 @@ var LibraryGLFW = {
   },
 
   glfwIconifyWindow: function() {
-    GLFW.iconifyWindow(GLFW.active.id);
+#if ASSERTIONS
+    warnOnce('glfwIconifyWindow is not implemented');
+#endif
   },
 
   glfwRestoreWindow: function() {
-    GLFW.restoreWindow(GLFW.active.id);
+#if ASSERTIONS
+    warnOnce('glfwRestoreWindow is not implemented');
+#endif
   },
 
   glfwSwapBuffers: function() {
@@ -1739,11 +1750,7 @@ var LibraryGLFW = {
   },
 
   glfwCreateThread: function(fun, arg) {
-    var str = 'v';
-    for (var i in arg) {
-      str += 'i';
-    }
-    dynCall(str, fun, arg);
+    {{{ makeDynCall('vi', 'str') }}}(fun, arg);
     // One single thread
     return 0;
   },
