@@ -1,7 +1,8 @@
-// Copyright 2011 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2011 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 //"use strict";
 
@@ -61,25 +62,42 @@ var LibraryManager = {
     // Core system libraries (always linked against)
     var libraries = [
       'library.js',
+      'library_stack.js',
       'library_formatString.js',
+      'library_math.js',
       'library_path.js',
       'library_signals.js',
       'library_syscall.js',
       'library_html5.js',
       'library_stack_trace.js',
-      'library_wasi.js'
+      'library_wasi.js',
+      'library_int53.js',
+      'library_dylink.js'
     ];
 
-    if (!DISABLE_EXCEPTION_THROWING) {
-      libraries.push('library_exceptions.js');
-    } else {
-      libraries.push('library_exceptions_stub.js');
+    if (!EXCEPTION_HANDLING) {
+      if (!DISABLE_EXCEPTION_THROWING) {
+        libraries.push('library_exceptions.js');
+      } else {
+        libraries.push('library_exceptions_stub.js');
+      }
     }
 
-    if (!MINIMAL_RUNTIME) {
+    if (MINIMAL_RUNTIME) {
+      // Classic runtime brings in string-related functions in the runtime preamble, by including
+      // runtime_strings_extra.js that contain the same contents as library_strings.js does. In
+      // MINIMAL_RUNTIME those string functions are available as JS library functions instead from
+      // library_strings.js, to avoid unconditionally bringing in extra code to the build.
+      libraries.push('library_strings.js');
+    } else {
       libraries.push('library_browser.js');
     }
 
+    if (USE_PTHREADS) { // TODO: Currently WebGL proxying makes pthreads library depend on WebGL.
+      libraries.push('library_webgl.js');
+      libraries.push('library_html5_webgl.js');
+    }
+
     if (FILESYSTEM) {
       // Core filesystem libraries (always linked against, unless -s FILESYSTEM=0 is specified)
       libraries = libraries.concat([
@@ -103,8 +121,8 @@ var LibraryManager = {
     if (AUTO_JS_LIBRARIES) {
       libraries = libraries.concat([
         'library_webgl.js',
+        'library_html5_webgl.js',
         'library_openal.js',
-        'library_vr.js',
         'library_sdl.js',
         'library_glut.js',
         'library_xlib.js',
@@ -116,14 +134,14 @@ var LibraryManager = {
         'library_async.js'
       ]);
     } else {
-      if (EMTERPRETIFY_ASYNC || ASYNCIFY) {
+      if (ASYNCIFY) {
         libraries.push('library_async.js');
       }
       if (USE_SDL == 1) {
         libraries.push('library_sdl.js');
       }
       if (USE_SDL == 2) {
-        libraries.push('library_egl.js', 'library_webgl.js');
+        libraries.push('library_egl.js', 'library_webgl.js', 'library_html5_webgl.js');
       }
     }
 
@@ -134,7 +152,7 @@ var LibraryManager = {
       libraries.push('library_lz4.js');
     }
 
-    if (USE_WEBGL2) {
+    if (MAX_WEBGL_VERSION >= 2) {
       libraries.push('library_webgl2.js');
     }
 
@@ -142,9 +160,24 @@ var LibraryManager = {
       libraries.push('library_glemu.js');
     }
 
-    if (BOOTSTRAPPING_STRUCT_INFO) libraries = ['library_bootstrap_structInfo.js', 'library_formatString.js'];
+    if (USE_WEBGPU) {
+      libraries.push('library_webgpu.js');
+      libraries.push('library_html5_webgpu.js');
+    }
 
-    // TODO: deduplicate libraries (not needed for correctness, but avoids unnecessary work)
+    if (BOOTSTRAPPING_STRUCT_INFO) {
+      libraries = [
+        'library_bootstrap.js',
+        'library_stack.js',
+        'library_formatString.js',
+        'library_stack_trace.js'
+      ];
+    }
+
+    // Deduplicate libraries to avoid processing any library file multiple times
+    libraries = libraries.filter(function(item, pos) {
+      return libraries.indexOf(item) == pos;
+    });
 
     // Save the list for has() queries later.
     this.libraries = libraries;
@@ -195,34 +228,41 @@ var LibraryManager = {
           if (lib[target].indexOf('Math_') == 0) continue libloop;
           target = lib[target];
         }
-        if (lib[target + '__asm']) continue; // This is an alias of an asm library function. Also needs to be fully optimized.
         if (!isNaN(target)) continue; // This is a number, and so cannot be an alias target.
         if (typeof lib[target] === 'undefined' || typeof lib[target] === 'function') {
-          // If the alias provides a signature, then construct a specific 'function foo(a0, a1, a2) { [return] _target(a0, a1, a2); }' form of forwarding.
-          // Otherwise construct a generic 'function foo() { return _target.apply(null, arguments); }' forwarding.
-          // The benefit of the first form is that Closure is able to fully inline and reason about the function.
-          // Note that the signature is checked on the alias function, not on the target function. That allows aliases to choose individually which form
-          // to use. 
-          if (lib[x + '__sig']) {
-            var argCount = lib[x + '__sig'].length - 1;
-            var ret = lib[x + '__sig'] == 'v' ? '' : 'return ';
-            var args = genArgSequence(argCount).join(',');
-            lib[x] = new Function(args, ret + '_' + target + '(' + args + ');');
-          } else {
-            lib[x] = new Function('return _' + target + '.apply(null, arguments)');
+          // When functions are aliased, the alias target must provide a signature for the function so that an efficient form of forwarding can be implemented.
+          // Primarily read the signature on the alias, and secondarily on the target.
+          function testStringType(sig) {
+            if (typeof lib[sig] !== 'undefined' && typeof typeof lib[sig] !== 'string') {
+              error(sig + ' should be a string! (was ' + typeof lib[sig]);
+            }
           }
+          testStringType(x + '__sig');
+          testStringType(target + '__sig');
+          if (typeof lib[x + '__sig'] === 'string' && typeof lib[target + '__sig'] === 'string' && lib[x + '__sig'] != lib[target + '__sig']) {
+            error(x + '__sig (' + lib[x + '__sig'] + ')  differs from ' + target + '__sig (' + lib[target + '__sig'] + ')');
+          }
+
+          var sig = lib[x + '__sig'] || lib[target + '__sig'];
+          if (typeof sig !== 'string') {
+            error('Function ' + x + ' aliases to target function ' + target + ', but neither the alias or the target provide a signature. Please add a ' + target + "__sig: 'vifj...' annotation or a " + x + "__sig: 'vifj...' annotation to describe the type of function forwarding that is needed!");
+          }
+
+          var argCount = sig.length - 1;
+          var ret = sig == 'v' ? '' : 'return ';
+          var args = genArgSequence(argCount).join(',');
+          lib[x] = new Function(args, ret + '_' + target + '(' + args + ');');
+
           if (!lib[x + '__deps']) lib[x + '__deps'] = [];
           lib[x + '__deps'].push(target);
         }
       }
     }
 
-    if (WASM_BACKEND) {
-      // all asm.js methods should just be run in JS. We should optimize them eventually into wasm. TODO
-      for (var x in lib) {
-        if (lib[x + '__asm']) {
-          lib[x + '__asm'] = undefined;
-        }
+    // all asm.js methods should just be run in JS. We should optimize them eventually into wasm. TODO
+    for (var x in lib) {
+      if (lib[x + '__asm']) {
+        lib[x + '__asm'] = undefined;
       }
     }
 
@@ -264,7 +304,6 @@ var LibraryManager = {
   },
 
   isStubFunction: function(ident) {
-    if (SIDE_MODULE == 1) return false; // cannot eliminate these, as may be implement in the main module and imported by us
     var libCall = LibraryManager.library[ident.substr(1)];
     return typeof libCall === 'function' && libCall.toString().replace(/\s/g, '') === 'function(){}'
                                          && !(ident in Functions.implementedFunctions);
@@ -305,10 +344,8 @@ function isExportedByForceFilesystem(name) {
          name === 'FS_createLink' ||
          name === 'FS_createDevice' ||
          name === 'FS_unlink' ||
-         name === 'getMemory' ||
          name === 'addRunDependency' ||
-         name === 'removeRunDependency' ||
-         name === 'calledRun';
+         name === 'removeRunDependency';
 }
 
 // export parts of the JS runtime that the user asked for
@@ -365,21 +402,11 @@ function exportRuntime() {
     'setValue',
     'getValue',
     'allocate',
-    'getMemory',
-    'AsciiToString',
-    'stringToAscii',
     'UTF8ArrayToString',
     'UTF8ToString',
     'stringToUTF8Array',
     'stringToUTF8',
     'lengthBytesUTF8',
-    'UTF16ToString',
-    'stringToUTF16',
-    'lengthBytesUTF16',
-    'UTF32ToString',
-    'stringToUTF32',
-    'lengthBytesUTF32',
-    'allocateUTF8',
     'stackTrace',
     'addOnPreRun',
     'addOnInit',
@@ -391,8 +418,6 @@ function exportRuntime() {
     'writeAsciiToMemory',
     'addRunDependency',
     'removeRunDependency',
-    'ENV',
-    'FS',
     'FS_createFolder',
     'FS_createPath',
     'FS_createDataFile',
@@ -401,10 +426,6 @@ function exportRuntime() {
     'FS_createLink',
     'FS_createDevice',
     'FS_unlink',
-    'GL',
-    'dynamicAlloc',
-    'loadDynamicLibrary',
-    'loadWebAssemblyModule',
     'getLEB',
     'getFunctionTables',
     'alignFunctionTables',
@@ -416,10 +437,6 @@ function exportRuntime() {
     'makeBigInt',
     'dynCall',
     'getCompilerSetting',
-    'stackSave',
-    'stackRestore',
-    'stackAlloc',
-    'establishStackSpace',
     'print',
     'printErr',
     'getTempRet0',
@@ -428,23 +445,58 @@ function exportRuntime() {
     'abort',
   ];
 
+  function isJsLibraryConfigIdentifier(ident) {
+    return ident.endsWith('__sig') || ident.endsWith('__proxy') || ident.endsWith('__asm') || ident.endsWith('__inline')
+     || ident.endsWith('__deps') || ident.endsWith('__postset') || ident.endsWith('__docs') || ident.endsWith('__import');
+  }
+
+  // Add JS library elements such as FS, GL, ENV, etc. These are prefixed with
+  // '$ which indicates they are JS methods.
+  for (var ident in LibraryManager.library) {
+    if (ident[0] === '$' && !isJsLibraryConfigIdentifier(ident)) {
+      runtimeElements.push(ident.substr(1));
+    }
+  }
+
   if (!MINIMAL_RUNTIME) {
-    runtimeElements.push('Pointer_stringify');
-    runtimeElements.push('warnOnce');
+    // MINIMAL_RUNTIME has moved these functions to library_strings.js
+    runtimeElements = runtimeElements.concat([
+      'warnOnce',
+      'stackSave',
+      'stackRestore',
+      'stackAlloc',
+      'AsciiToString',
+      'stringToAscii',
+      'UTF16ToString',
+      'stringToUTF16',
+      'lengthBytesUTF16',
+      'UTF32ToString',
+      'stringToUTF32',
+      'lengthBytesUTF32',
+      'allocateUTF8',
+      'allocateUTF8OnStack'
+    ]);
+
   }
 
   if (STACK_OVERFLOW_CHECK) {
     runtimeElements.push('writeStackCookie');
     runtimeElements.push('checkStackCookie');
-    runtimeElements.push('abortStackOverflow');
   }
 
   if (USE_PTHREADS) {
     // In pthreads mode, the following functions always need to be exported to
     // Module for closure compiler, and also for MODULARIZE (so worker.js can
     // access them).
-    ['PThread', 'ExitStatus', 'tempDoublePtr', 'wasmMemory', '_pthread_self',
-     'ExitStatus', 'tempDoublePtr'].forEach(function(x) {
+    var threadExports = ['PThread', '_pthread_self'];
+    if (WASM) {
+      threadExports.push('wasmMemory');
+    }
+    if (!MINIMAL_RUNTIME) {
+      threadExports.push('ExitStatus');
+    }
+
+    threadExports.forEach(function(x) {
       EXPORTED_RUNTIME_METHODS_SET[x] = 1;
       runtimeElements.push(x);
     });
@@ -466,9 +518,7 @@ function exportRuntime() {
   var runtimeNumbers = [
     'ALLOC_NORMAL',
     'ALLOC_STACK',
-    'ALLOC_DYNAMIC',
     'ALLOC_NONE',
-    'calledRun',
   ];
   if (ASSERTIONS) {
     // check all exported things exist, warn about typos
@@ -491,8 +541,6 @@ var PassManager = {
     print('\n//FORWARDED_DATA:' + JSON.stringify({
       Functions: Functions,
       EXPORTED_FUNCTIONS: EXPORTED_FUNCTIONS,
-      STATIC_BUMP: STATIC_BUMP, // updated with info from JS
-      DYNAMICTOP_PTR: DYNAMICTOP_PTR,
       ATINITS: ATINITS.join('\n'),
       ATMAINS: ATMAINS.join('\n'),
       ATEXITS: ATEXITS.join('\n'),
