@@ -377,7 +377,6 @@ function makeGlobalUse(ident) {
       return ident;
     }
     var ret = (Runtime.GLOBAL_BASE + index).toString();
-    if (SIDE_MODULE) ret = '(H_BASE+' + ret + ')';
     return ret;
   }
   return ident;
@@ -629,7 +628,7 @@ function parseNumerical(value, type) {
       if (value[0] === '-' && ret === 0) { return '-.0'; } // fix negative 0, toString makes it 0
     }
     if (type === 'double' || type === 'float') {
-      if (!RUNNING_JS_OPTS) ret = asmEnsureFloat(ret, type);
+      ret = asmEnsureFloat(ret, type);
     }
     return ret.toString();
   } else {
@@ -748,8 +747,7 @@ function ensureDot(value) {
   value = value.toString();
   // if already dotted, or Infinity or NaN, nothing to do here
   // if smaller than 1 and running js opts, we always need to force a coercion (0.001 will turn into 1e-3, which has no .)
-  if ((value.indexOf('.') >= 0 || /[IN]/.test(value)) && (!RUNNING_JS_OPTS || Math.abs(value) >= 1)) return value;
-  if (RUNNING_JS_OPTS) return '(+' + value + ')'; // JS optimizer will run, we must do +x, and it will be corrected later
+  if ((value.indexOf('.') >= 0 || /[IN]/.test(value))) return value;
   var e = value.indexOf('e');
   if (e < 0) return value + '.0';
   return value.substr(0, e) + '.0' + value.substr(e);
@@ -773,7 +771,7 @@ function asmEnsureFloat(value, type) { // ensures that a float type has either 5
 function asmInitializer(type) {
   if (type in Compiletime.FLOAT_TYPES) {
     if (type === 'float') return 'Math_fround(0)';
-    return RUNNING_JS_OPTS ? '+0' : '.0';
+    return '.0';
   } else {
     return '0';
   }
@@ -910,7 +908,7 @@ function makeGetValueAsm(ptr, pos, type, unsigned) {
 //!             'null' means, in the context of SAFE_HEAP, that we should accept all types;
 //!             which means we should write to all slabs, ignore type differences if any on reads, etc.
 //! @param noNeedFirst Whether to ignore the offset in the pointer itself.
-function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign, forceAsm) {
+function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign) {
   sep = sep || ';';
   if (isStructType(type)) {
     var typeData = Types.types[type];
@@ -976,45 +974,8 @@ function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe,
   return makeGetSlabs(ptr, type, true).map(function(slab) { return slab + '[' + getHeapOffset(offset, type) + ']=' + value }).join(sep);
 }
 
-function makeSetValueAsm(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign) {
-  return makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign, true);
-}
-
 var UNROLL_LOOP_MAX = 8;
 
-function makeSetValues(ptr, pos, value, type, num, align) {
-  function unroll(type, num, jump, value$) {
-    jump = jump || 1;
-    value$ = value$ || value;
-    return range(num).map(function(i) {
-      return makeSetValue(ptr, getFastValue(pos, '+', i*jump), value$, type);
-    }).join('; ');
-  }
-  // If we don't know how to handle this at compile-time, or handling it is best done in a large amount of code, call memset
-  // TODO: optimize the case of numeric num but non-numeric value
-  if (!isNumber(num) || !isNumber(value) || (parseInt(num)/align >= UNROLL_LOOP_MAX)) {
-    return '_memset(' + asmCoercion(getFastValue(ptr, '+', pos), 'i32') + ', ' + asmCoercion(value, 'i32') + ', ' + asmCoercion(num, 'i32') + ')|0';
-  }
-  num = parseInt(num);
-  value = parseInt(value);
-  if (value < 0) value += 256; // make it unsigned
-  var values = {
-    1: value,
-    2: value | (value << 8),
-    4: value | (value << 8) | (value << 16) | (value << 24)
-  };
-  var ret = [];
-  [4, 2, 1].forEach(function(possibleAlign) {
-    if (num == 0) return;
-    if (align >= possibleAlign) {
-      ret.push(unroll('i' + (possibleAlign*8), Math.floor(num/possibleAlign), possibleAlign, values[possibleAlign]));
-      pos = getFastValue(pos, '+', Math.floor(num/possibleAlign)*possibleAlign);
-      num %= possibleAlign;
-    }
-  });
-  return ret.join('; ');
-}
-
 var TYPED_ARRAY_SET_MIN = Infinity; // .set() as memcpy seems to just slow us down
 
 function makeCopyValues(dest, src, num, type, modifier, align, sep) {
@@ -1056,17 +1017,13 @@ function makeHEAPView(which, start, end) {
 // When dynamically linking, some things like dynCalls may not exist in one module and
 // be provided by a linked module, so they must be accessed indirectly using Module
 function exportedAsmFunc(func) {
-  if (!MAIN_MODULE && !SIDE_MODULE) {
+  if (!MAIN_MODULE) {
     return func;
   } else {
     return "Module['" + func + "']";
   }
 }
 
-function makeDynCall(sig) {
-  return exportedAsmFunc('dynCall_' + sig);
-}
-
 var TWO_TWENTY = Math.pow(2, 20);
 
 // Given two values and an operation, returns the result of that operation.
@@ -1261,9 +1218,6 @@ function makePointer(slab, pos, allocator, type, ptr, finalMemoryInitialization)
       // writing out into memory, without a normal allocation. We put all of these into a single big chunk.
       assert(typeof slab == 'object');
       assert(slab.length % QUANTUM_SIZE == 0, slab.length); // must be aligned already
-      if (SIDE_MODULE && typeof ptr == 'string') {
-        ptr = parseInt(ptr.substring(ptr.indexOf('+'), ptr.length-1)); // parse into (H_BASE+X)
-      }
       var offset = ptr - Runtime.GLOBAL_BASE;
       for (var i = 0; i < slab.length; i++) {
         memoryInitialization[offset + i] = slab[i];
@@ -1462,8 +1416,13 @@ function asmFFICoercion(value, type) {
   return value;
 }
 
-function makeDynCall(sig) {
-  return 'dynCall_' + sig;
+function makeDynCall(sig, funcPtr) {
+  assert(sig.indexOf('j') == -1);
+  if (USE_LEGACY_DYNCALLS) {
+    return `getDynCaller("${sig}", ${funcPtr})`;
+  } else {
+    return `wasmTable.get(${funcPtr})`;
+  }
 }
 
 function heapAndOffset(heap, ptr) { // given   HEAP8, ptr   , we return    splitChunk, relptr
@@ -1485,19 +1444,6 @@ function makeEval(code) {
   return ret;
 }
 
-function makeStaticAlloc(size) {
-  size = alignMemory(size);
-  var ret = alignMemory(GLOBAL_BASE + STATIC_BUMP);
-  STATIC_BUMP = ret + size - GLOBAL_BASE;
-  return ret;
-}
-
-function makeStaticString(string) {
-  var len = lengthBytesUTF8(string) + 1;
-  var ptr = makeStaticAlloc(len);
-  return '(stringToUTF8("' + string + '", ' + ptr + ', ' + len + '), ' + ptr + ')';
-}
-
 var ATINITS = [];
 
 function addAtInit(code) {
@@ -1546,11 +1492,6 @@ function makeRetainedCompilerSettings() {
 // In wasm, the heap size must be a multiple of 64KB.
 // In asm.js, it must be a multiple of 16MB.
 var WASM_PAGE_SIZE = 65536;
-var ASMJS_PAGE_SIZE = 16777216;
-
-function getMemoryPageSize() {
-  return WASM ? WASM_PAGE_SIZE : ASMJS_PAGE_SIZE;
-}
 
 // Page size reported by some POSIX calls, mostly filesystem. This does not
 // depend on the memory page size which differs between wasm and asm.js, and
@@ -1729,3 +1670,18 @@ function addReadyPromiseAssertions(promise) {
     `;
   }).join('\n');
 }
+
+function makeMalloc(source, param) {
+  if ('_malloc' in IMPLEMENTED_FUNCTIONS) {
+    return '_malloc(' + param + ')';
+  }
+  // It should be impossible to call some functions without malloc being
+  // included, unless we have a deps_info.json bug. To let closure not error
+  // on `_malloc` not being present, they don't call malloc and instead abort
+  // with an error at runtime.
+  // TODO: A more comprehensive deps system could catch this at compile time.
+  if (!ASSERTIONS) {
+    return "abort();";
+  }
+  return `abort('malloc was not included, but is needed in ${source}. Adding "_malloc" to EXPORTED_FUNCTIONS should fix that. This may be a bug in the compiler, please file an issue.');`
+}
