@@ -10,14 +10,9 @@ function writeStackCookie() {
 #if ASSERTIONS
   assert((STACK_MAX & 3) == 0);
 #endif
-#if WASM_BACKEND
   // The stack grows downwards
   HEAPU32[(STACK_MAX >> 2)+1] = 0x2135467;
   HEAPU32[(STACK_MAX >> 2)+2] = 0x89BACDFE;
-#else
-  HEAPU32[(STACK_MAX >> 2)-1] = 0x2135467;
-  HEAPU32[(STACK_MAX >> 2)-2] = 0x89BACDFE;
-#endif
 #if !USE_ASAN
   // Also test the global address 0 for integrity.
   // We don't do this with ASan because ASan does its own checks for this.
@@ -26,13 +21,8 @@ function writeStackCookie() {
 }
 
 function checkStackCookie() {
-#if WASM_BACKEND
   var cookie1 = HEAPU32[(STACK_MAX >> 2)+1];
   var cookie2 = HEAPU32[(STACK_MAX >> 2)+2];
-#else
-  var cookie1 = HEAPU32[(STACK_MAX >> 2)-1];
-  var cookie2 = HEAPU32[(STACK_MAX >> 2)-2];
-#endif
   if (cookie1 != 0x2135467 || cookie2 != 0x89BACDFE) {
     abort('Stack overflow! Stack cookie has been overwritten, expected hex dwords 0x89BACDFE and 0x2135467, but received 0x' + cookie2.toString(16) + ' ' + cookie1.toString(16));
   }
